!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
!
! !MODULE: SMOPSsm_Mod
! 
! !DESCRIPTION: 
!   This module contains interfaces and subroutines to
!   handle 
! 
! !REVISION HISTORY: 
!  8 May 2013    Sujay Kumar; initial specification
! 
module SMOPSsm_Mod
! !USES: 
  use ESMF
  use map_utils

  implicit none

  PRIVATE

!-----------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!-----------------------------------------------------------------------------
  public :: SMOPSsm_setup
!-----------------------------------------------------------------------------
! !PUBLIC TYPES:
!-----------------------------------------------------------------------------
  public :: SMOPSsm_struc
!EOP
  type, public:: SMOPSsm_dec
     
     integer                :: useRealtime
     integer                :: useASCAT
     integer                :: useWindSat
     integer                :: useSsdevScal
     logical                :: startMode
     integer                :: nc
     integer                :: nr
     real,     allocatable      :: smobs(:,:)
     real,     allocatable      :: smtime(:,:)

     real                       :: ssdev_inp
     integer, allocatable       :: n11(:)

     real,    allocatable       :: model_xrange(:,:,:)
     real,    allocatable       :: obs_xrange(:,:,:)
     real,    allocatable       :: model_cdf(:,:,:)
     real,    allocatable       :: obs_cdf(:,:,:)
     real,    allocatable       :: model_mu(:,:)
     real,    allocatable       :: obs_mu(:,:)
     real,    allocatable       :: model_sigma(:,:)
     real,    allocatable       :: obs_sigma(:,:)

     integer                :: nbins
     integer                :: ntimes
  end type SMOPSsm_dec
  
  type(SMOPSsm_dec),allocatable :: SMOPSsm_struc(:)
  
contains

!BOP
! 
! !ROUTINE: SMOPSsm_setup
! \label{SMOPSsm_setup}
! 
! !INTERFACE: 
  subroutine SMOPSsm_setup(k, OBS_State, OBS_Pert_State)
! !USES: 
    use ESMF
    use LIS_coreMod
    use LIS_timeMgrMod
    use LIS_historyMod
    use LIS_dataAssimMod
    use LIS_perturbMod
    use LIS_logmod

    implicit none 

! !ARGUMENTS: 
    integer                ::  k
    type(ESMF_State)       ::  OBS_State(LIS_rc%nnest)
    type(ESMF_State)       ::  OBS_Pert_State(LIS_rc%nnest)
! 
! !DESCRIPTION: 
!   
!   This routine completes the runtime initializations and 
!   creation of data strctures required for handling SMOPS 
!   soil moisture data. 
!  
!   The arguments are: 
!   \begin{description}
!    \item[OBS\_State]   observation state 
!    \item[OBS\_Pert\_State] observation perturbations state
!   \end{description}
!EOP
    real, parameter        ::  minssdev =0.04 
    integer                ::  n,i,t,kk,jj
    integer                ::  ftn
    integer                ::  status
    type(ESMF_Field)       ::  obsField(LIS_rc%nnest)
    type(ESMF_ArraySpec)   ::  intarrspec, realarrspec
    type(ESMF_Field)       ::  pertField(LIS_rc%nnest)
    type(ESMF_ArraySpec)   ::  pertArrSpec
    character*100          ::  rtsmopssmobsdir
    character*100          ::  temp
    real,  allocatable         ::  obsstd(:)
    character*1            ::  vid(2)
    character*40, allocatable  ::  vname(:)
    real        , allocatable  ::  varmin(:)
    real        , allocatable  ::  varmax(:)
    type(pert_dec_type)    ::  obs_pert
    real, pointer          ::  obs_temp(:,:)
    real                   :: gridDesci(50)
    character*100          :: modelcdffile(LIS_rc%nnest)
    character*100          :: obscdffile(LIS_rc%nnest)
    real, allocatable          :: ssdev(:)

    integer                :: c,r
    real, allocatable          :: ssdev_grid(:,:)

    allocate(SMOPSsm_struc(LIS_rc%nnest))

    call ESMF_ArraySpecSet(intarrspec,rank=1,typekind=ESMF_TYPEKIND_I4,&
         rc=status)
    call LIS_verify(status)

    call ESMF_ArraySpecSet(realarrspec,rank=1,typekind=ESMF_TYPEKIND_R4,&
         rc=status)
    call LIS_verify(status)

    call ESMF_ArraySpecSet(pertArrSpec,rank=2,typekind=ESMF_TYPEKIND_R4,&
         rc=status)
    call LIS_verify(status)

    call ESMF_ConfigFindLabel(LIS_config,"SMOPS soil moisture data directory:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,rtsmopssmobsdir,&
            rc=status)
       call LIS_verify(status, 'SMOPS soil moisture data directory: is missing')

       call ESMF_AttributeSet(OBS_State(n),"Data Directory",&
            rtsmopssmobsdir, rc=status)
       call LIS_verify(status)
    enddo


    call ESMF_ConfigFindLabel(LIS_config,"SMOPS use realtime data:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,SMOPSsm_struc(n)%useRealtime,&
            rc=status)
       call LIS_verify(status, 'SMOPS use realtime data: is missing')

    enddo

    call ESMF_ConfigFindLabel(LIS_config,"SMOPS soil moisture use ASCAT data:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,SMOPSsm_struc(n)%useASCAT,&
            rc=status)
       call LIS_verify(status, 'SMOPS soil moisture use ASCAT data: is missing')
       
    enddo

    call ESMF_ConfigFindLabel(LIS_config,"SMOPS soil moisture use WindSat data:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,SMOPSsm_struc(n)%useWindSat,&
            rc=status)
       call LIS_verify(status, 'SMOPS soil moisture use WindSat data: is missing')
       
    enddo
    
    call ESMF_ConfigFindLabel(LIS_config,"SMOPS soil moisture use scaled standard deviation model:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,SMOPSsm_struc(n)%useSsdevScal,&
            rc=status)
       call LIS_verify(status, 'SMOPS soil moisture use scaled standard deviation model: is missing')
       
    enddo
    call ESMF_ConfigFindLabel(LIS_config,"SMOPS model CDF file:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,modelcdffile(n),rc=status)
       call LIS_verify(status, 'SMOPS model CDF file: not defined')
    enddo

    call ESMF_ConfigFindLabel(LIS_config,"SMOPS observation CDF file:",&
         rc=status)
    do n=1,LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,obscdffile(n),rc=status)
       call LIS_verify(status, 'SMOPS observation CDF file: not defined')
    enddo
    
    call ESMF_ConfigFindLabel(LIS_config, "SMOPS soil moisture number of bins in the CDF:", rc=status)
    do n=1, LIS_rc%nnest
       call ESMF_ConfigGetAttribute(LIS_config,SMOPSsm_struc(n)%nbins, rc=status)
       call LIS_verify(status, "SMOPS soil moisture number of bins in the CDF: not defined")
    enddo

   do n=1,LIS_rc%nnest
       call ESMF_AttributeSet(OBS_State(n),"Data Update Status",&
            .false., rc=status)
       call LIS_verify(status)

       call ESMF_AttributeSet(OBS_State(n),"Data Update Time",&
            -99.0, rc=status)
       call LIS_verify(status)

       call ESMF_AttributeSet(OBS_State(n),"Data Assimilate Status",&
            .false., rc=status)
       call LIS_verify(status)
       
       call ESMF_AttributeSet(OBS_State(n),"Number Of Observations",&
            LIS_rc%ngrid(n),rc=status)
       call LIS_verify(status)
       
    enddo

    write(LIS_logunit,*)'read SMOPS soil moisture data specifications'       

!----------------------------------------------------------------------------
!   Create the array containers that will contain the observations and
!   the perturbations. SMOPS
!   observations are in the grid space. Since there is only one layer
!   being assimilated, the array size is LIS_rc%ngrid(n). 
!   
!----------------------------------------------------------------------------

    do n=1,LIS_rc%nnest
       
       write(unit=temp,fmt='(i2.2)') 1
       read(unit=temp,fmt='(2a1)') vid

       obsField(n) = ESMF_FieldCreate(arrayspec=realarrspec,&
            grid=LIS_vecGrid(n),&
            name="Observation"//vid(1)//vid(2),rc=status)
       call LIS_verify(status)

!Perturbations State
       write(LIS_logunit,*) 'Opening attributes for observations ',&
            trim(LIS_rc%obsattribfile(k))
       ftn = LIS_getNextUnitNumber()
       open(ftn,file=trim(LIS_rc%obsattribfile(k)),status='old')
       read(ftn,*)
       read(ftn,*) LIS_rc%nobtypes(k)
       read(ftn,*)
    
       allocate(vname(LIS_rc%nobtypes(k)))
       allocate(varmax(LIS_rc%nobtypes(k)))
       allocate(varmin(LIS_rc%nobtypes(k)))
       
       do i=1,LIS_rc%nobtypes(k)
          read(ftn,fmt='(a40)') vname(i)
          read(ftn,*) varmin(i),varmax(i)
          write(LIS_logunit,*) vname(i),varmin(i),varmax(i)
       enddo
       call LIS_releaseUnitNumber(ftn)  
       
       allocate(ssdev(LIS_rc%ngrid(n)))
       
       if(trim(LIS_rc%perturb_obs(k)).ne."none") then 
          allocate(obs_pert%vname(1))
          allocate(obs_pert%perttype(1))
          allocate(obs_pert%ssdev(1))
          allocate(obs_pert%stdmax(1))
          allocate(obs_pert%zeromean(1))
          allocate(obs_pert%tcorr(1))
          allocate(obs_pert%xcorr(1))
          allocate(obs_pert%ycorr(1))
          allocate(obs_pert%ccorr(1,1))

          call LIS_readPertAttributes(1,LIS_rc%obspertAttribfile(k),&
               obs_pert)

! Set obs err to be uniform (will be rescaled later for each grid point). 
          ssdev = obs_pert%ssdev(1)
          SMOPSsm_struc(n)%ssdev_inp = obs_pert%ssdev(1)

          pertField(n) = ESMF_FieldCreate(arrayspec=pertArrSpec,&
               grid=LIS_ensOnGrid(n),name="Observation"//vid(1)//vid(2),&
               rc=status)
          call LIS_verify(status)
          
! initializing the perturbations to be zero 
          call ESMF_FieldGet(pertField(n),localDE=0,farrayPtr=obs_temp,rc=status)
          call LIS_verify(status)
          obs_temp(:,:) = 0 

          call ESMF_AttributeSet(pertField(n),"Perturbation Type",&
               obs_pert%perttype(1), rc=status)
          call LIS_verify(status)

          if(LIS_rc%ngrid(n).gt.0) then 
             call ESMF_AttributeSet(pertField(n),"Standard Deviation",&
                  ssdev,itemCount=LIS_rc%ngrid(n),rc=status)
             call LIS_verify(status)
          endif
          
          call ESMF_AttributeSet(pertField(n),"Std Normal Max",&
               obs_pert%stdmax(1), rc=status)
          call LIS_verify(status)
          
          call ESMF_AttributeSet(pertField(n),"Ensure Zero Mean",&
               obs_pert%zeromean(1),rc=status)
          call LIS_verify(status)
          
          call ESMF_AttributeSet(pertField(n),"Temporal Correlation Scale",&
               obs_pert%tcorr(1),rc=status)
          call LIS_verify(status)
          
          call ESMF_AttributeSet(pertField(n),"X Correlation Scale",&
               obs_pert%xcorr(1),rc=status)
          
          call ESMF_AttributeSet(pertField(n),"Y Correlation Scale",&
               obs_pert%ycorr(1),rc=status)

          call ESMF_AttributeSet(pertField(n),"Cross Correlation Strength",&
               obs_pert%ccorr(1,:),itemCount=1,rc=status)

       endif
          
       deallocate(vname)
       deallocate(varmax)
       deallocate(varmin)
       deallocate(ssdev)

    enddo
    write(LIS_logunit,*) &
         'Created the States to hold the SMOPS observations data'
    do n=1,LIS_rc%nnest
       SMOPSsm_struc(n)%nc = 1440
       SMOPSsm_struc(n)%nr = 720
       allocate(SMOPSsm_struc(n)%smobs(LIS_rc%lnc(n),LIS_rc%lnr(n)))
       allocate(SMOPSsm_struc(n)%smtime(LIS_rc%lnc(n),LIS_rc%lnr(n)))

    enddo
    
    do n=1,LIS_rc%nnest
       allocate(ssdev(LIS_rc%ngrid(n)))
       ssdev = obs_pert%ssdev(1)
       
       call LIS_getTemporalCDFres(modelcdffile(n),&
            SMOPSsm_struc(n)%ntimes)
       
       allocate(SMOPSsm_struc(n)%model_xrange(&
            LIS_rc%ngrid(n), SMOPSsm_struc(n)%ntimes, &
            SMOPSsm_struc(n)%nbins))
       allocate(SMOPSsm_struc(n)%obs_xrange(&
            LIS_rc%ngrid(n), SMOPSsm_struc(n)%ntimes, &
            SMOPSsm_struc(n)%nbins))
       allocate(SMOPSsm_struc(n)%model_cdf(&
            LIS_rc%ngrid(n), SMOPSsm_struc(n)%ntimes, &
            SMOPSsm_struc(n)%nbins))
       allocate(SMOPSsm_struc(n)%obs_cdf(&
            LIS_rc%ngrid(n), SMOPSsm_struc(n)%ntimes, &
            SMOPSsm_struc(n)%nbins))
       allocate(SMOPSsm_struc(n)%model_mu(LIS_rc%ngrid(n),&
            SMOPSsm_struc(n)%ntimes))
       allocate(SMOPSsm_struc(n)%model_sigma(LIS_rc%ngrid(n),&
            SMOPSsm_struc(n)%ntimes))
       allocate(SMOPSsm_struc(n)%obs_mu(LIS_rc%ngrid(n),&
            SMOPSsm_struc(n)%ntimes))
       allocate(SMOPSsm_struc(n)%obs_sigma(LIS_rc%ngrid(n),&
            SMOPSsm_struc(n)%ntimes))

!----------------------------------------------------------------------------
! Read the model and observation CDF data
!----------------------------------------------------------------------------
       call LIS_readMeanSigmaData(n,&
            SMOPSsm_struc(n)%ntimes,&
            modelcdffile(n), &
            "SoilMoist",&
            SMOPSsm_struc(n)%model_mu,&
            SMOPSsm_struc(n)%model_sigma)
       
       call LIS_readMeanSigmaData(n,&
            SMOPSsm_struc(n)%ntimes,&
            obscdffile(n), &
            "SoilMoist",&
            SMOPSsm_struc(n)%obs_mu,&
            SMOPSsm_struc(n)%obs_sigma)
       
       call LIS_readCDFdata(n,&
            SMOPSsm_struc(n)%nbins,&
            SMOPSsm_struc(n)%ntimes,&
            modelcdffile(n), &
            "SoilMoist",&
            SMOPSsm_struc(n)%model_xrange,&
            SMOPSsm_struc(n)%model_cdf)
       
       call LIS_readCDFdata(n,&
            SMOPSsm_struc(n)%nbins,&
            SMOPSsm_struc(n)%ntimes,&
            obscdffile(n), &
            "SoilMoist",&
            SMOPSsm_struc(n)%obs_xrange,&
            SMOPSsm_struc(n)%obs_cdf)
       
!       allocate(ssdev_grid(LIS_rc%lnc(n),LIS_rc%lnr(n)))
!       ssdev_grid = LIS_rc%udef

       if(SMOPSsm_struc(n)%useSsdevScal.eq.1) then 
          if(SMOPSsm_struc(n)%ntimes.eq.1) then 
             jj = 1
          else
             jj = LIS_rc%mo
          endif
          do t=1,LIS_rc%ngrid(n)
             if(SMOPSsm_struc(n)%obs_sigma(t,jj).gt.0) then 
                ssdev(t) = ssdev(t)*SMOPSsm_struc(n)%model_sigma(t,jj)/&
                     SMOPSsm_struc(n)%obs_sigma(t,jj)
!                c = LIS_domain(n)%grid(t)%col
!                r = LIS_domain(n)%grid(t)%row
!                ssdev_grid(c,r) = ssdev(t)             
                if(ssdev(t).lt.minssdev) then 
                   ssdev(t) = minssdev
                endif
             endif
          enddo
          
!          open(100,file='ssdev.bin',form='unformatted')
!          write(100) ssdev_grid
!          close(100)
!          stop
       endif

       if(LIS_rc%ngrid(n).gt.0) then 
          call ESMF_AttributeSet(pertField(n),"Standard Deviation",&
               ssdev,itemCount=LIS_rc%ngrid(n),rc=status)
          call LIS_verify(status)
       endif

       deallocate(ssdev)
    enddo
    
    do n=1,LIS_rc%nnest
       gridDesci = 0 
       gridDesci(1) = 0 
       gridDesci(2) = 1440
       gridDesci(3) = 720
       gridDesci(4) = -89.875
       gridDesci(5) = -179.875
       gridDesci(6) = 128
       gridDesci(7) = 89.875
       gridDesci(8) = 179.875
       gridDesci(9) = 0.25
       gridDesci(10) = 0.25
       gridDesci(20) = 64
       
       allocate(SMOPSsm_struc(n)%n11(LIS_rc%lnc(n)*LIS_rc%lnr(n)))
       
       call neighbor_interp_input(n,gridDesci,&
            SMOPSsm_struc(n)%n11)
       
       call LIS_registerAlarm("SMOPS read alarm",&
            86400.0, 86400.0)
       SMOPSsm_struc(n)%startMode = .true. 

       call ESMF_StateAdd(OBS_State(n),(/obsField(n)/),rc=status)
       call LIS_verify(status)
     
       call ESMF_StateAdd(OBS_Pert_State(n),(/pertField(n)/),rc=status)
       call LIS_verify(status)

    enddo
  end subroutine SMOPSsm_setup
end module SMOPSsm_Mod
