!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
! !ROUTINE: clsmf25_qc_soilmobs
! \label{clsmf25_qc_soilmobs}
!
! !REVISION HISTORY:
! 25Feb2008: Sujay Kumar: Initial Specification
!
! !INTERFACE:
subroutine clsmf25_qc_soilmobs(n,OBS_State)
! !USES:
  use ESMF
  use LIS_coreMod
  use LIS_logMod
  use LIS_constantsMod, only : LIS_CONST_TKFRZ
  use LIS_vegDataMod, only : LIS_lai,LIS_gfrac
  use clsmf25_lsmMod

  implicit none
! !ARGUMENTS: 
  integer, intent(in)      :: n
  type(ESMF_State)         :: OBS_State
!
! !DESCRIPTION:
!
!  This subroutine performs any model-based QC of the observation 
!  prior to data assimilation. Here the soil moisture observations
!  are flagged when LSM indicates that (1) rain is falling (2)
!  soil is frozen or (3) ground is fully or partially covered 
!  with snow. 
!  
!  The arguments are: 
!  \begin{description}
!  \item[n] index of the nest \newline
!  \item[OBS\_State] ESMF state container for observations \newline
!  \end{description}
!
!EOP
  type(ESMF_Field)         :: obs_sm_field

  real, pointer            :: smobs(:)
  integer                  :: t
  integer                  :: gid
  integer                  :: status
  real                     :: lat,lon

  call ESMF_StateGet(OBS_State,"Observation01",obs_sm_field,&
       rc=status)
  call LIS_verify(status,&
       "ESMF_StateGet failed in clsmf25_qc_soilmobs")
  call ESMF_FieldGet(obs_sm_field,localDE=0,farrayPtr=smobs,rc=status)
  call LIS_verify(status,& 
       "ESMF_FieldGet failed in clsmf25_qc_soilmobs")

  do t = 1,LIS_rc%npatch(n,LIS_rc%lsm_index)

     gid  = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%index
     lat =  LIS_domain(n)%grid(gid)%lat
     lon =  LIS_domain(n)%grid(gid)%lon

     if(smobs(gid).ne.LIS_rc%udef) then 
        if(clsmf25_struc(n)%met_force(t)%rainf.gt.3E-6) then 
           smobs(gid) = LIS_rc%udef
        endif
        if((clsmf25_struc(n)%cat_diagn(t)%tp(1)+273.16).le.LIS_CONST_TKFRZ) then
           smobs(gid) = LIS_rc%udef
        endif
        if((clsmf25_struc(n)%cat_diagn(t)%tp(2)+273.16).le.LIS_CONST_TKFRZ) then
           smobs(gid) = LIS_rc%udef
        endif
        if((clsmf25_struc(n)%cat_diagn(t)%tp(3)+273.16).le.LIS_CONST_TKFRZ) then
           smobs(gid) = LIS_rc%udef
        endif
        if(clsmf25_struc(n)%cat_diagn(t)%tsurf.le.LIS_CONST_TKFRZ) then
           smobs(gid) = LIS_rc%udef
        endif
        if(LIS_surface(n,LIS_rc%lsm_index)%tile(t)%vegt.le.3) then !forest types
           smobs(gid) = LIS_rc%udef
        endif
        if((clsmf25_struc(n)%cat_progn(t)%wesn(1) +         &
             clsmf25_struc(n)%cat_progn(t)%wesn(2) +         &
             clsmf25_struc(n)%cat_progn(t)%wesn(3)).gt.0.0001) then 
           smobs(gid) = LIS_rc%udef
        endif
!        if(clsmf25_struc(n)%cat_output(t)%snocovr.gt.0.0) then 
!           smobs(gid) = LIS_rc%udef
!        endif
!        if(LIS_gfrac(n)%greenness(t).gt.0.90) then 
!           smobs(gid) = LIS_rc%udef        
!        endif
        if(LIS_lai(n)%tlai(t).gt.2) then 
           smobs(gid) = LIS_rc%udef        
        endif
        if(abs(smobs(gid)- clsmf25_struc(n)%cat_param(t)%poros).lt.0.05) then 
           smobs(gid) = LIS_rc%udef
        endif
     endif

  enddo

end subroutine clsmf25_qc_soilmobs
   
